# -*- coding: utf-8 -*-
# ========================================================================
#  2022/6/15 20:31
#                 _____   _   _   _____   __   _   _____  
#                /  ___| | | | | | ____| |  \ | | /  ___| 
#                | |     | |_| | | |__   |   \| | | |     
#                | |     |  _  | |  __|  | |\   | | |  _  
#                | |___  | | | | | |___  | | \  | | |_| | 
#                \_____| |_| |_| |_____| |_|  \_| \_____/ 
# ------------------------------------------------------------------------
# 
# 
# 
# ========================================================================

import os
import time

import joblib
import sys
sys.path.append('../../')

import configparser


def check_path(path):
    if not os.path.exists(path):
        os.makedirs(path)


def get_pred_class(_target='total_energy'):
    if _target == 'total_energy':
        from predict_structure_mpi import PredictStructureMPI
        return PredictStructureMPI
    elif _target == 'formation_energy':
        from predict_structure_mpi_by_formation_energy import FindMaxFormationEnergyMaterialMPI
        return FindMaxFormationEnergyMaterialMPI
    elif _target == 'binding_energy':
        from predict_structure_mpi_by_binding_energy import FindMaxBindingEnergyMaterialMPI
        return FindMaxBindingEnergyMaterialMPI
    else:
        raise RuntimeError('Target class not exist!')


def get_config(_output_path='.', _storage=None, **kwargs):
    _config = configparser.ConfigParser()
    _config.add_section('BASE')
    # _config.set('BASE', 'compound', 'Si32')
    # _config.set('BASE', 'atom_element', '[55,82,53] [55,82,53] [55,82,53]')
    # _config.set('BASE', 'atom_count', '[1-10] [1-10] [1-10]')
    _config.set('BASE', 'atom_element', '[5-7] [5-7]')
    _config.set('BASE', 'atom_count', '[1-4] [1-4]')
    _config.set('BASE', 'use_children_cell', 'True')
    _config.set('BASE', 'min_atomic_dist_limit', '-0.2')
    _config.set('BASE', 'volume_limit', '[0, 0]')
    _config.set('BASE', 'max_vacuum_limit', '7.0')
    _config.set('BASE', 'output_path', _output_path)

    _config.add_section('CALCULATOR')
    _config.set('CALCULATOR', 'calculator', 'm3gnet')
    _config.set('CALCULATOR', 'calculator_path', r'F:\d3ream\calculators\m3gnet\origin_model\EFS2021')
    _config.set('CALCULATOR', 'use_calculator_relax', 'True')
    _config.set('CALCULATOR', 'use_keep_symmetry', 'True')
    _config.set('CALCULATOR', 'symprec', '0.001')
    _config.set('CALCULATOR', 'use_gpu', 'False')

    _config.add_section('OPTIMIZER')
    _config.set('OPTIMIZER', 'algorithm', "tpe2")
    _config.set('OPTIMIZER', 'n_init', '40')
    _config.set('OPTIMIZER', 'max_step', '6000')
    _config.set('OPTIMIZER', 'rand_seed', '100')
    # _config.set('OPTIMIZER', 'use_resume', 'False')
    _config.set('OPTIMIZER', 'n_mpi', '1')
    _config.set('OPTIMIZER', 'storage', _storage)

    _config.add_section('LATTICE')
    _config.set('LATTICE', 'space_group', '[1-230]')
    _config.set('LATTICE', 'use_flexible_site', 'True')
    _config.set('LATTICE', 'lattice_a', '[2.0-30.0]')
    _config.set('LATTICE', 'lattice_b', '[2.0-30.0]')
    _config.set('LATTICE', 'lattice_c', '[2.0-30.0]')
    _config.set('LATTICE', 'lattice_alpha', '[20.0-160.0]')
    _config.set('LATTICE', 'lattice_beta', '[20.0-160.0]')
    _config.set('LATTICE', 'lattice_gamma', '[20.0-160.0]')

    return _config


if __name__ == '__main__':
    # target = 'total_energy'
    # target = 'formation_energy'
    target = 'binding_energy'
    target_class = get_pred_class(target)

    n_mpi = 6
    # study_name = target + '_13'
    study_name = target + '_15'
    _output_path = './results/' + study_name

    output_path = os.path.join(_output_path, 'results')
    structures_path = os.path.join(output_path, 'structures')
    check_path(output_path)
    check_path(structures_path)

    from test.my_password import storage
    config = get_config(_output_path, storage)

    # optuna create-study --study-name "test" --storage "sqlite:///test.db"
    # optuna create-study --study-name "test" --storage "mysql://"
    os.system('''optuna create-study --study-name "%s" --storage "%s" --skip-if-exists ''' % (study_name, storage))
    time.sleep(10)

    ####################################################
    # using mpirun, recommend using in cluster machine #
    ####################################################
    # target_class(None, config, study_name)

    ##################################################
    # using joblib, recommend using in local machine #
    ##################################################
    with joblib.parallel_backend('loky'):
        joblib.Parallel(verbose=100)(
            joblib.delayed(target_class)(None, config, study_name)
            for _ in range(n_mpi)
        )
