# -*- coding: utf-8 -*-
# ========================================================================
#  2022/5/25 22:30
#                 _____   _   _   _____   __   _   _____  
#                /  ___| | | | | | ____| |  \ | | /  ___| 
#                | |     | |_| | | |__   |   \| | | |     
#                | |     |  _  | |  __|  | |\   | | |  _  
#                | |___  | | | | | |___  | | \  | | |_| | 
#                \_____| |_| |_| |_____| |_|  \_| \_____/ 
# ------------------------------------------------------------------------
# 
# 
# 
# ========================================================================

import sys
sys.path.append('../')

import configparser
from predict_structure import PredictStructure


def input_config(comp):
    config = configparser.ConfigParser()
    config.add_section('BASE')
    config.set('BASE', 'compound', comp)
    config.set('BASE', 'use_children_cell', 'True')
    config.set('BASE', 'min_atomic_dist_limit', '-0.6')
    config.set('BASE', 'volume_limit', '[0, 0]')
    config.set('BASE', 'max_vacuum_limit', '7.0')
    config.set('BASE', 'output_path', './CSP_test/' + comp.replace(' ', ''))

    config.add_section('CALCULATOR')
    config.set('CALCULATOR', 'calculator', 'm3gnet')
    config.set('CALCULATOR', 'calculator_path', r'F:\d3ream\calculators\m3gnet\origin_model\EFS2021')
    config.set('CALCULATOR', 'use_calculator_relax', 'True')
    config.set('CALCULATOR', 'use_keep_symmetry', 'False')
    config.set('CALCULATOR', 'symprec', '0.001')
    config.set('CALCULATOR', 'use_gpu', 'False')

    config.add_section('OPTIMIZER')
    config.set('OPTIMIZER', 'algorithm', "tpe2")
    config.set('OPTIMIZER', 'n_init', '20')
    config.set('OPTIMIZER', 'max_step', '500')
    config.set('OPTIMIZER', 'rand_seed', '100')
    config.set('OPTIMIZER', 'use_resume', 'False')
    config.set('OPTIMIZER', 'n_mpi', '1')
    config.set('OPTIMIZER', 'storage', '')

    config.add_section('LATTICE')
    config.set('LATTICE', 'space_group', '[%d-%d]' % (14, 14))
    # config.set('LATTICE', 'use_flexible_site', 'False')
    config.set('LATTICE', 'wyck_pos_gen', '1')
    config.set('LATTICE', 'max_wyck_pos_count', '200000')
    config.set('LATTICE', 'lattice_a', '[%f-%f]' % (2.0, 30.0))
    config.set('LATTICE', 'lattice_b', '[%f-%f]' % (2.0, 30.0))
    config.set('LATTICE', 'lattice_c', '[%f-%f]' % (2.0, 30.0))
    config.set('LATTICE', 'lattice_alpha', '[%f-%f]' % (20.0, 160.0))
    config.set('LATTICE', 'lattice_beta', '[%f-%f]' % (20.0, 160.0))
    config.set('LATTICE', 'lattice_gamma', '[%f-%f]' % (20.0, 160.0))

    return config


if __name__ == '__main__':
    # compounds = [
    #     "Li4 F4", "Na4 F4", "K4 F4", "Rb4 F4", "Cs4 F4",
    #     "Li4 Cl4", "Na4 Cl4", "K4 Cl4", "Rb4 Cl4", "Cs4 Cl4",
    #     "Be4 O4", "Mg4 O4", "Ca4 O4", "Sr4 O4", "Ba4 O4", "Zn4 O4", "Cd4 O4",
    #     "Be4 S4", "Mg4 S4", "Ca4 S4", "Sr4 S4", "Ba4 S4", "Zn4 S4", "Cd4 S4",
    #     'C8', 'Si8',
    #     'Ga4 As4', 'Cd4 Te4',
    #     'Cs1 Pb1 I3', 'Cs2 Pb2 I6',
    # ]
    compounds = [
        # 'C8',
        # 'Si8',
        # 'Cd4 Te4',
        # 'Cs10 Pb10 I30',
        # 'Cd4 Te4',
        # 'C32',
        # 'Ag4 F8',
        # 'Si20',
        # 'Zr3 C2',
        # 'Si4 V4',
        # 'Al8 As8',
        # 'Mg8 Br8',
        # 'Zr4 C4',
        'Zr4 O8',
        # 'Zr8 O16',
    ]
    # compounds = [
    #     # 'Zn8 O8',
    #     # 'In8 Cu8 Se16'
    #     # 'Be4 C4 N8'
    #     # 'Ti4 O8',
    #     # "Li4 Cl4",
    #     # "Li4 Br4",
    #     # "Li4 I4",
    # ]

    # is_parallel = True
    is_parallel = False

    if is_parallel:
        import multiprocessing
        pool = multiprocessing.Pool(processes=4)
        for compound in compounds:
            c_config = input_config(compound)
            pool.apply_async(PredictStructure, args=(None, c_config))
        pool.close()
        pool.join()
    else:
        for compound in compounds:
            c_config = input_config(compound)
            PredictStructure(None, c_config)
